@@ -19,6 +19,7 @@
 #include <iterator>
 #include <tuple>
 #include <cassert>
+#include <type_traits>
 
 #include "onedpl_config.h"
 #include "utils.h"
@@ -75,6 +76,38 @@ struct __make_references
     }
 };
 
+template <typename _Iter, typename = void>
+struct __is_legacy_passed_directly : std::false_type
+{
+};
+
+template <typename _Iter>
+struct __is_legacy_passed_directly<_Iter, std::enable_if_t<_Iter::is_passed_directly::value>> : std::true_type
+{
+};
+
+template <typename _T>
+struct __is_reversed_indirectly_device_accessible_it;
+
+template <typename T>
+constexpr auto is_onedpl_indirectly_device_accessible(T)
+    -> std::disjunction<
+#if _ONEDPL_BACKEND_SYCL
+        oneapi::dpl::__internal::__is_known_usm_vector_iter<std::decay_t<T>>,             // USM vector iterator
+#endif // _ONEDPL_BACKEND_SYCL
+        std::is_pointer<std::decay_t<T>>,                                                 // USM pointer
+        oneapi::dpl::__internal::__is_legacy_passed_directly<std::decay_t<T>>,            // legacy passed directly iter
+        oneapi::dpl::__internal::__is_reversed_indirectly_device_accessible_it<std::decay_t<T>>>; // reverse iterator
+
+struct __is_onedpl_indirectly_device_accessible_fn
+{
+    template <typename T>
+    constexpr auto
+    operator()(T t) const -> decltype(is_onedpl_indirectly_device_accessible(t));
+};
+
+inline constexpr __is_onedpl_indirectly_device_accessible_fn __is_onedpl_indirectly_device_accessible;
+
 //zip_iterator version for forward iterator
 //== and != comparison is performed only on the first element of the tuple
 //
@@ -94,14 +127,14 @@ class zip_forward_iterator
 #endif
 
     static const ::std::size_t __num_types = sizeof...(_Types);
-    typedef __tuple_t<_Types...> __it_types;
+    using __it_types = __tuple_t<_Types...>;
 
   public:
-    typedef ::std::make_signed_t<::std::size_t> difference_type;
-    typedef __tuple_t<typename ::std::iterator_traits<_Types>::value_type...> value_type;
-    typedef __tuple_t<typename ::std::iterator_traits<_Types>::reference...> reference;
-    typedef __tuple_t<typename ::std::iterator_traits<_Types>::pointer...> pointer;
-    typedef ::std::forward_iterator_tag iterator_category;
+    using difference_type = std::make_signed_t<std::size_t>;
+    using value_type = __tuple_t<typename std::iterator_traits<_Types>::value_type...>;
+    using reference = __tuple_t<typename std::iterator_traits<_Types>::reference...>;
+    using pointer = __tuple_t<typename std::iterator_traits<_Types>::pointer...>;
+    using iterator_category = std::forward_iterator_tag;
 
     zip_forward_iterator() : __my_it_() {}
     explicit zip_forward_iterator(_Types... __args) : __my_it_(__tuple_t<_Types...>{__args...}) {}
@@ -157,18 +190,33 @@ namespace oneapi
 {
 namespace dpl
 {
+
+template <typename T>
+struct is_indirectly_device_accessible
+    : decltype(oneapi::dpl::__internal::__is_onedpl_indirectly_device_accessible(std::declval<T>()))
+{
+    static_assert(std::is_same_v<decltype(decltype(oneapi::dpl::__internal::__is_onedpl_indirectly_device_accessible(
+                                     std::declval<T>()))::value),
+                                 const bool>,
+                  "Return type of is_onedpl_indirectly_device_accessible does not have the characteristics of a "
+                  "bool_constant");
+};
+
+template <typename T>
+inline constexpr bool is_indirectly_device_accessible_v = is_indirectly_device_accessible<T>::value;
+
 template <typename _Ip>
 class counting_iterator
 {
     static_assert(::std::is_integral_v<_Ip>, "Cannot instantiate counting_iterator with a non-integer type");
 
   public:
-    typedef ::std::make_signed_t<_Ip> difference_type;
-    typedef _Ip value_type;
-    typedef const _Ip* pointer;
+    using difference_type = std::make_signed_t<_Ip>;
+    using value_type = _Ip;
+    using pointer = const _Ip*;
     // There is no storage behind the iterator, so we return a value instead of reference.
-    typedef _Ip reference;
-    typedef ::std::random_access_iterator_tag iterator_category;
+    using reference = _Ip;
+    using iterator_category = std::random_access_iterator_tag;
 
     counting_iterator() : __my_counter_() {}
     explicit counting_iterator(_Ip __init) : __my_counter_(__init) {}
@@ -266,6 +314,12 @@ class counting_iterator
         return !(*this < __it);
     }
 
+    friend std::true_type
+    is_onedpl_indirectly_device_accessible(counting_iterator)
+    {
+        return {}; //minimal body provided to avoid warnings of non-template-friend in g++
+    }
+
   private:
     _Ip __my_counter_;
 };
@@ -275,14 +329,14 @@ class zip_iterator
 {
     static_assert(sizeof...(_Types) > 0, "Cannot instantiate zip_iterator with empty template parameter pack");
     static const ::std::size_t __num_types = sizeof...(_Types);
-    typedef oneapi::dpl::__internal::tuple<_Types...> __it_types;
+    using __it_types = oneapi::dpl::__internal::tuple<_Types...>;
 
   public:
-    typedef ::std::make_signed_t<::std::size_t> difference_type;
-    typedef oneapi::dpl::__internal::tuple<typename ::std::iterator_traits<_Types>::value_type...> value_type;
-    typedef oneapi::dpl::__internal::tuple<typename ::std::iterator_traits<_Types>::reference...> reference;
-    typedef ::std::tuple<typename ::std::iterator_traits<_Types>::pointer...> pointer;
-    typedef ::std::random_access_iterator_tag iterator_category;
+    using difference_type = std::make_signed_t<std::size_t>;
+    using value_type = oneapi::dpl::__internal::tuple<typename std::iterator_traits<_Types>::value_type...>;
+    using reference = oneapi::dpl::__internal::tuple<typename std::iterator_traits<_Types>::reference...>;
+    using pointer = std::tuple<typename std::iterator_traits<_Types>::pointer...>;
+    using iterator_category = std::random_access_iterator_tag;
     using is_zip = ::std::true_type;
 
     zip_iterator() : __my_it_() {}
@@ -398,6 +452,13 @@ class zip_iterator
         return !(*this < __it);
     }
 
+    friend auto
+    is_onedpl_indirectly_device_accessible(zip_iterator)
+        -> std::conjunction<oneapi::dpl::is_indirectly_device_accessible<_Types>...>
+    {
+        return {}; //minimal body provided to avoid warnings of non-template-friend in g++
+    }
+
   private:
     __it_types __my_it_;
 };
@@ -428,11 +489,11 @@ class transform_iterator
                   "base iterator as argument.");
 
   public:
-    typedef typename ::std::iterator_traits<_Iter>::difference_type difference_type;
-    typedef decltype(__my_unary_func_(::std::declval<typename ::std::iterator_traits<_Iter>::reference>())) reference;
-    typedef ::std::remove_reference_t<reference> value_type;
-    typedef typename ::std::iterator_traits<_Iter>::pointer pointer;
-    typedef typename ::std::iterator_traits<_Iter>::iterator_category iterator_category;
+    using difference_type = typename std::iterator_traits<_Iter>::difference_type;
+    using reference = decltype(__my_unary_func_(std::declval<typename std::iterator_traits<_Iter>::reference>()));
+    using value_type = std::remove_reference_t<reference>;
+    using pointer = typename std::iterator_traits<_Iter>::pointer;
+    using iterator_category = typename std::iterator_traits<_Iter>::iterator_category;
 
     //default constructor will only be present if both the unary functor and iterator are default constructible
     transform_iterator() = default;
@@ -574,6 +635,11 @@ class transform_iterator
     {
         return __my_unary_func_;
     }
+    friend auto
+    is_onedpl_indirectly_device_accessible(transform_iterator) -> oneapi::dpl::is_indirectly_device_accessible<_Iter>
+    {
+        return {}; //minimal body provided to avoid warnings of non-template-friend in g++
+    }
 };
 
 template <typename _Iter, typename _UnaryFunc>
@@ -607,18 +673,17 @@ template <typename SourceIterator, typename _Permutation>
 class permutation_iterator
 {
   public:
-    typedef std::conditional_t<
+    using IndexMap = std::conditional_t<
         !__internal::__is_functor<_Permutation>, _Permutation,
         transform_iterator<counting_iterator<typename ::std::iterator_traits<SourceIterator>::difference_type>,
-                           _Permutation>>
-        IndexMap;
-    typedef typename ::std::iterator_traits<SourceIterator>::difference_type difference_type;
-    typedef typename ::std::iterator_traits<SourceIterator>::value_type value_type;
-    typedef typename ::std::iterator_traits<SourceIterator>::pointer pointer;
-    typedef typename ::std::iterator_traits<SourceIterator>::reference reference;
-    typedef SourceIterator base_type;
-    typedef ::std::random_access_iterator_tag iterator_category;
-    typedef ::std::true_type is_permutation;
+                           _Permutation>>;
+    using difference_type = typename std::iterator_traits<SourceIterator>::difference_type;
+    using value_type = typename std::iterator_traits<SourceIterator>::value_type;
+    using pointer = typename std::iterator_traits<SourceIterator>::pointer;
+    using reference = typename std::iterator_traits<SourceIterator>::reference;
+    using base_type = SourceIterator;
+    using iterator_category = std::random_access_iterator_tag;
+    using is_permutation = std::true_type;
 
     permutation_iterator() = default;
 
@@ -765,6 +830,14 @@ class permutation_iterator
         return !(*this < it);
     }
 
+    friend auto
+    is_onedpl_indirectly_device_accessible(permutation_iterator)
+        -> std::conjunction<oneapi::dpl::is_indirectly_device_accessible<SourceIterator>,
+                            oneapi::dpl::is_indirectly_device_accessible<permutation_iterator::IndexMap>>
+    {
+        return {}; //minimal body provided to avoid warnings of non-template-friend in g++
+    }
+
   private:
     SourceIterator my_source_it;
     IndexMap my_index;
@@ -815,11 +888,11 @@ inline constexpr ignore_copyable ignore{};
 class discard_iterator
 {
   public:
-    typedef ::std::ptrdiff_t difference_type;
-    typedef internal::ignore_copyable value_type;
-    typedef void* pointer;
-    typedef value_type reference;
-    typedef ::std::random_access_iterator_tag iterator_category;
+    using difference_type = std::ptrdiff_t;
+    using value_type = internal::ignore_copyable;
+    using pointer = void*;
+    using reference = value_type;
+    using iterator_category = std::random_access_iterator_tag;
     using is_discard = ::std::true_type;
 
     discard_iterator() : __my_position_() {}
@@ -927,6 +1000,12 @@ class discard_iterator
         return !(*this < __it);
     }
 
+    friend std::true_type
+    is_onedpl_indirectly_device_accessible(discard_iterator)
+    {
+        return {}; //minimal body provided to avoid warnings of non-template-friend in g++
+    }
+
   private:
     difference_type __my_position_;
 };
@@ -940,6 +1019,16 @@ namespace dpl
 {
 namespace __internal
 {
+template <typename _T>
+struct __is_reversed_indirectly_device_accessible_it : std::false_type
+{
+};
+
+template <typename _BaseIter>
+struct __is_reversed_indirectly_device_accessible_it<std::reverse_iterator<_BaseIter>>
+    : oneapi::dpl::is_indirectly_device_accessible<_BaseIter>
+{
+};
 
 struct make_zipiterator_functor
 {
