@@ -34,8 +34,8 @@ struct test_one_policy
                Predicate pred)
     {
         const bool expected = ::std::lexicographical_compare(begin1, end1, begin2, end2, pred);
-        const bool actual = ::std::lexicographical_compare(exec, begin1, end1, begin2, end2, pred);
-        EXPECT_TRUE(actual == expected, "wrong return result from lexicographical compare with predicate");
+        const bool actual = std::lexicographical_compare(std::forward<ExecutionPolicy>(exec), begin1, end1, begin2, end2, pred);
+        EXPECT_EQ(expected, actual, "wrong return result from lexicographical compare with predicate");
     }
 
     template <typename ExecutionPolicy, typename Iterator1, typename Iterator2>
@@ -43,8 +43,8 @@ struct test_one_policy
     operator()(ExecutionPolicy&& exec, Iterator1 begin1, Iterator1 end1, Iterator2 begin2, Iterator2 end2)
     {
         const bool expected = ::std::lexicographical_compare(begin1, end1, begin2, end2);
-        const bool actual = ::std::lexicographical_compare(exec, begin1, end1, begin2, end2);
-        EXPECT_TRUE(actual == expected, "wrong return result from lexicographical compare without predicate");
+        const bool actual = std::lexicographical_compare(std::forward<ExecutionPolicy>(exec), begin1, end1, begin2, end2);
+        EXPECT_EQ(expected, actual, "wrong return result from lexicographical compare without predicate");
     }
 };
 
@@ -53,29 +53,29 @@ void
 test(Predicate pred)
 {
 
-    const ::std::size_t max_n = 1000000;
-    Sequence<T1> in1(max_n, [](::std::size_t k) { return T1(k); });
-    Sequence<T2> in2(2 * max_n, [](::std::size_t k) { return T2(k); });
+    const ::std::size_t max_size = 1000000;
+    Sequence<T1> in1(max_size, [](::std::size_t k) { return T1(k); });
+    Sequence<T2> in2(2 * max_size, [](::std::size_t k) { return T2(k); });
 
     ::std::size_t n2;
 
     // Test case: Call algorithm's version without predicate.
-    invoke_on_all_policies<0>()(test_one_policy<T1>(), in1.cbegin(), in1.cbegin() + max_n,
-                                in2.cbegin() + 3 * max_n / 10, in2.cbegin() + 5 * max_n / 10);
+    invoke_on_all_policies<0>()(test_one_policy<T1>(), in1.cbegin(), in1.cbegin() + max_size,
+                                in2.cbegin() + 3 * max_size / 10, in2.cbegin() + 5 * max_size / 10);
 
     // Test case: If one range is a prefix of another, the shorter range is lexicographically less than the other.
-    ::std::size_t max_n2 = max_n / 10;
-    invoke_on_all_policies<1>()(test_one_policy<T1>(), in1.begin(), in1.begin() + max_n, in2.cbegin(),
-                                in2.cbegin() + max_n2, pred);
-    invoke_on_all_policies<2>()(test_one_policy<T1>(), in1.begin(), in1.begin() + max_n, in2.begin() + max_n2,
-                                in2.begin() + 3 * max_n2, pred);
+    ::std::size_t max_size2 = max_size / 10;
+    invoke_on_all_policies<1>()(test_one_policy<T1>(), in1.begin(), in1.begin() + max_size, in2.cbegin(),
+                                in2.cbegin() + max_size2, pred);
+    invoke_on_all_policies<2>()(test_one_policy<T1>(), in1.begin(), in1.begin() + max_size, in2.begin() + max_size2,
+                                in2.begin() + 3 * max_size2, pred);
 
     // Test case: If one range is a prefix of another, the shorter range is lexicographically less than the other.
-    max_n2 = 2 * max_n;
-    invoke_on_all_policies<3>()(test_one_policy<T1>(), in1.cbegin(), in1.cbegin() + max_n, in2.begin(),
-                                in2.begin() + max_n2, pred);
+    max_size2 = 2 * max_size;
+    invoke_on_all_policies<3>()(test_one_policy<T1>(), in1.cbegin(), in1.cbegin() + max_size, in2.begin(),
+                                in2.begin() + max_size2, pred);
 
-    for (::std::size_t n1 = 0; n1 <= max_n; n1 = n1 <= 16 ? n1 + 1 : ::std::size_t(3.1415 * n1))
+    for (::std::size_t n1 = 0; n1 <= max_size; n1 = n1 <= 16 ? n1 + 1 : ::std::size_t(3.1415 * n1))
     {
         // Test case: If two ranges have equivalent elements and are of the same length, then the ranges are lexicographically equal.
         n2 = n1;
@@ -103,16 +103,15 @@ template <typename Predicate>
 void
 test_string(Predicate pred)
 {
-
-    const ::std::size_t max_n = 1000000;
+    const std::size_t max_size = 1000000;
     ::std::string in1 = "";
     ::std::string in2 = "";
-    for (::std::size_t n1 = 0; n1 <= max_n; ++n1)
+    for (::std::size_t n1 = 0; n1 <= max_size; ++n1)
     {
         in1 += n1;
     }
 
-    for (::std::size_t n1 = 0; n1 <= 2 * max_n; ++n1)
+    for (std::size_t n1 = 0; n1 <= 2 * max_size; ++n1)
     {
         in2 += n1;
     }
@@ -137,8 +136,8 @@ test_string(Predicate pred)
         invoke_on_all_policies<9>()(test_one_policy<Predicate>(), in1.begin(), in1.begin() + n1, in2.cbegin(),
                                     in2.cbegin() + n2, pred);
     }
-    invoke_on_all_policies<10>()(test_one_policy<Predicate>(), in1.cbegin(), in1.cbegin() + max_n,
-                                 in2.cbegin() + 3 * max_n / 10, in2.cbegin() + 5 * max_n / 10);
+    invoke_on_all_policies<10>()(test_one_policy<Predicate>(), in1.cbegin(), in1.cbegin() + max_size,
+                                 in2.cbegin() + 3 * max_size / 10, in2.cbegin() + 5 * max_size / 10);
 }
 
 template <typename T>
@@ -162,7 +161,7 @@ struct test_non_const
     void
     operator()(Policy&& exec, FirstIterator first_iter, SecondInterator second_iter)
     {
-        lexicographical_compare(exec, first_iter, first_iter, second_iter, second_iter, non_const(::std::less<T>()));
+        lexicographical_compare(std::forward<Policy>(exec), first_iter, first_iter, second_iter, second_iter, non_const(std::less<T>()));
     }
 };
 
