@@ -42,7 +42,7 @@ DEFINE_TEST(test_transform_inclusive_scan)
     {
         TestDataTransfer<UDTKind::eKeys, Size> host_keys(*this, n);
 
-        typedef typename std::iterator_traits<Iterator1>::value_type T1;
+        using T1 = typename std::iterator_traits<Iterator1>::value_type;
 
         std::fill(host_keys.get(), host_keys.get() + n, T1(1));
         host_keys.update_data();
