@@ -23,6 +23,7 @@
 #include "oneapi/dpl/iterator"
 
 #include "support/utils.h"
+#include "support/utils_invoke.h" // CLONE_TEST_POLICY_IDX
 #include "support/scan_serial_impl.h"
 
 using namespace TestUtils;
@@ -151,7 +152,7 @@ DEFINE_TEST_1(test_scan_non_inplace, TestingAlgoritm)
             is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator1>>
     operator()(Policy&& exec,
                Iterator1 keys_first, Iterator1 keys_last,
-               Iterator2 vals_first, Iterator2 vals_last,
+               Iterator2 vals_first, Iterator2 /*vals_last*/,
                Size n)
     {
         using ValT = typename ::std::iterator_traits<Iterator2>::value_type;
@@ -177,7 +178,7 @@ DEFINE_TEST_1(test_scan_non_inplace, TestingAlgoritm)
         update_data(host_keys);
 
         // Now we are ready to call the tested algorithm
-        testingAlgo.call_onedpl(make_new_policy<new_kernel_name<Policy, 0>>(exec), keys_first, keys_last, vals_first);
+        testingAlgo.call_onedpl(CLONE_TEST_POLICY_IDX(exec, 0), keys_first, keys_last, vals_first);
 
         // After the tested algorithm finished we should check the results.
         // For that, at first we copy data from the buffer described by iterators
@@ -198,7 +199,7 @@ DEFINE_TEST_1(test_scan_non_inplace, TestingAlgoritm)
             is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator1>>
     operator()(Policy&& exec,
                Iterator1 keys_first, Iterator1 keys_last,
-               Iterator2 vals_first, Iterator2 vals_last,
+               Iterator2 vals_first, Iterator2 /*vals_last*/,
                Size n)
     {
         using ValT = typename ::std::iterator_traits<Iterator2>::value_type;
@@ -208,7 +209,7 @@ DEFINE_TEST_1(test_scan_non_inplace, TestingAlgoritm)
         // Initialize source data in the buffer [keys_first, keys_last)
         initialize_data(keys_first, n);
 
-        testingAlgo.call_onedpl(exec, keys_first, keys_last, vals_first);
+        testingAlgo.call_onedpl(std::forward<Policy>(exec), keys_first, keys_last, vals_first);
 
         std::vector<ValT> expected(n);
         testingAlgo.call_serial(keys_first, keys_last + n, expected.data());
@@ -218,10 +219,7 @@ DEFINE_TEST_1(test_scan_non_inplace, TestingAlgoritm)
     // specialization for non-random_access iterators
     template <typename Policy, typename Iterator1, typename Iterator2, typename Size>
     ::std::enable_if_t<!is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator1>>
-    operator()(Policy&& exec,
-               Iterator1 keys_first, Iterator1 keys_last,
-               Iterator2 vals_first, Iterator2 vals_last,
-               Size n)
+    operator()(Policy&&, Iterator1, Iterator1, Iterator2, Iterator2, Size)
     {
     }
 };
@@ -246,7 +244,7 @@ DEFINE_TEST_1(test_scan_inplace, TestingAlgoritm)
         const std::vector<KeyT> source_host_keys_state(keys_first, keys_first + n);
 
         // Now we are ready to call the tested algorithm
-        testingAlgo.call_onedpl(exec, keys_first, keys_last, keys_first);
+        testingAlgo.call_onedpl(std::forward<Policy>(exec), keys_first, keys_last, keys_first);
 
         std::vector<KeyT> expected(n);
         testingAlgo.call_serial(source_host_keys_state.cbegin(), source_host_keys_state.cend(), expected.data());
@@ -275,7 +273,7 @@ DEFINE_TEST_1(test_scan_inplace, TestingAlgoritm)
         update_data(host_keys);
 
         // Now we are ready to call tested algorithm
-        testingAlgo.call_onedpl(make_new_policy<new_kernel_name<Policy, 0>>(exec), keys_first, keys_last, keys_first);
+        testingAlgo.call_onedpl(CLONE_TEST_POLICY_IDX(exec, 0), keys_first, keys_last, keys_first);
 
         retrieve_data(host_keys);
 
@@ -287,7 +285,7 @@ DEFINE_TEST_1(test_scan_inplace, TestingAlgoritm)
     // specialization for non-random_access iterators
     template <typename Policy, typename Iterator1, typename Size>
     ::std::enable_if_t<!is_base_of_iterator_category_v<::std::random_access_iterator_tag, Iterator1>>
-    operator()(Policy&& exec, Iterator1 keys_first, Iterator1 keys_last, Size n)
+    operator()(Policy&&, Iterator1, Iterator1, Size)
     {
     }
 };
