@@ -22,16 +22,18 @@
 #endif
 
 #include "support/utils.h"
+#include "support/utils_invoke.h" // for CLONE_TEST_POLICY macro
 
 #include <cstdint>
 #if _ONEDPL_DEBUG_SYCL
 #include <iostream>
 #endif
 
-std::int32_t
-main()
-{
 #if _ENABLE_RANGES_TESTING
+template <typename Policy>
+void
+test_impl(Policy&& exec)
+{
     const int n = 7, n_res = 4;
     int a[n] = {1, 3, 3, 3, 2, 2, 1}; // input keys
     int b[n] = {9, 8, 7, 6, 5, 4, 3}; // input values
@@ -41,27 +43,24 @@ main()
     sycl::buffer<int> C(n);           // output keys
     sycl::buffer<int> D(n);           // output values
 
-    using namespace TestUtils;
     using namespace oneapi::dpl::experimental::ranges;
 
-    auto exec = TestUtils::default_dpcpp_policy;
-
-    [[maybe_unused]] auto res = reduce_by_segment(exec, views::all_read(A), views::all_read(B),
+    [[maybe_unused]] auto res = reduce_by_segment(CLONE_TEST_POLICY(exec), views::all_read(A), views::all_read(B),
                                                   views::all_write(C), views::all_write(D));
 
     int key_exp[n_res] = {1, 3, 2, 1};    // expected keys
     int value_exp[n_res] = {9, 21, 9, 3}; // expected values
 
 #if _ONEDPL_DEBUG_SYCL
-    ::std::cout << "keys: ";
-    for(auto v: views::host_all(C) | views::take(res))
-        ::std::cout << v << " ";
-    ::std::cout << ::std::endl;
+    std::cout << "keys: ";
+    for (auto v : views::host_all(C) | views::take(res))
+        std::cout << v << " ";
+    std::cout << std::endl;
 
-    ::std::cout << "values: ";
-    for(auto v: views::host_all(D) | views::take(res))
-        ::std::cout << v << " ";
-    ::std::cout << ::std::endl;
+    std::cout << "values: ";
+    for (auto v : views::host_all(D) | views::take(res))
+        std::cout << v << " ";
+    std::cout << std::endl;
 #endif // _ONEDPL_DEBUG_SYCL
 
     //check result
@@ -69,11 +68,24 @@ main()
     EXPECT_EQ_N(value_exp, views::host_all(D).begin(), n_res, "wrong values from reduce_by_segment");
 
 // Check if a kernel name can be omitted when a compiler supports implicit names
-#if __SYCL_UNNAMED_LAMBDA__
-    sycl::buffer<::std::uint64_t> E(n);
-    reduce_by_segment(exec, views::all_read(A), views::all_read(B), views::all_write(C), views::all_write(E));
-#endif
+#if __SYCL_UNNAMED_LAMBDA__ && !TEST_EXPLICIT_KERNEL_NAMES
+    sycl::buffer<std::uint64_t> E(n);
+    reduce_by_segment(CLONE_TEST_POLICY(exec), views::all_read(A), views::all_read(B), views::all_write(C), views::all_write(E));
+#endif // __SYCL_UNNAMED_LAMBDA__ && !TEST_EXPLICIT_KERNEL_NAMES
+}
+#endif // _ENABLE_RANGES_TESTING
 
+std::int32_t
+main()
+{
+#if _ENABLE_RANGES_TESTING
+
+    auto policy = TestUtils::get_dpcpp_test_policy();
+    test_impl(policy);
+
+#if TEST_CHECK_COMPILATION_WITH_DIFF_POLICY_VAL_CATEGORY
+    TestUtils::check_compilation(policy, [](auto&& policy) { test_impl(std::forward<decltype(policy)>(policy)); });
+#endif
 #endif //_ENABLE_RANGES_TESTING
 
     return TestUtils::done(_ENABLE_RANGES_TESTING);
