@@ -26,10 +26,11 @@
 
 #include <iostream>
 
-std::int32_t
-main()
-{
 #if _ENABLE_RANGES_TESTING
+template <typename Policy>
+void
+test_impl(Policy&& exec)
+{
     constexpr int max_n = 10;
 
     using namespace oneapi::dpl::experimental::ranges;
@@ -39,14 +40,31 @@ main()
     auto iota = views::iota(0, max_n);
     //the name nano::ranges::copy is not injected into oneapi::dpl::experimental::ranges namespace
     __nanorange::nano::ranges::copy(iota, views::host_all(A).begin());
-    reverse(TestUtils::default_dpcpp_policy, A);
+    reverse(std::forward<Policy>(exec), A);
 
+#if _ONEDPL_DEBUG_SYCL
     for(auto v: views::host_all(A))
-        ::std::cout << v << " ";
-    ::std::cout << ::std::endl;
+        std::cout << v << " ";
+    std::cout << std::endl;
+#endif
+
     //check result
     EXPECT_EQ_RANGES(iota | views::reverse, views::host_all(A), "wrong effect from reverse");
+}
+#endif // _ENABLE_RANGES_TESTING
 
+std::int32_t
+main()
+{
+#if _ENABLE_RANGES_TESTING
+
+    auto policy = TestUtils::get_dpcpp_test_policy();
+    test_impl(policy);
+
+#if TEST_CHECK_COMPILATION_WITH_DIFF_POLICY_VAL_CATEGORY
+    TestUtils::check_compilation(policy, [](auto&& policy) { test_impl(std::forward<decltype(policy)>(policy)); });
+#endif
 #endif //_ENABLE_RANGES_TESTING
+
     return TestUtils::done(_ENABLE_RANGES_TESTING);
 }
