@@ -47,8 +47,12 @@ namespace test_std_ranges
 template <typename>
 constexpr bool supress_dangling_iterators_check = false;
 
+#if PSTL_USE_DEBUG
+inline constexpr int big_size = (1<<22) + 37; //4M
+#else
 // The largest specializations of algorithms with device policies handle 16M+ elements.
 inline constexpr int big_size = (1<<24) + 10; //16M
+#endif
 
 // ~100K is sufficient for parallel policies.
 // It also usually results in using multiple-work-group specializations for device policies.
@@ -81,6 +85,7 @@ auto f = [](auto&& val) { return val * val; };
 auto binary_f = [](auto&& val1, auto&& val2) { return val1 * val2; };
 auto proj = [](auto&& val){ return val * 2; };
 auto pred = [](auto&& val) { return val == 5; };
+auto select_many = [](int val) { return (val % 29) > 1; };
 
 auto binary_pred = [](auto&& val1, auto&& val2) { return val1 == val2; };
 auto binary_pred_const = [](const auto& val1, const auto& val2) { return val1 == val2; };
@@ -193,12 +198,12 @@ constexpr int calc_res_size(int n, int) { return n; }
 // If in1 range is empty, then the out range is always empty
 // Can be specialized with an algorithm type if the behaviour is different, e.g. see set_union test.
 template<typename>
-int out_size_with_empty_in1(int) { return 0; };
+int out_size_with_empty_in1(int) { return 0; }
 
 // If in2 range is empty, then the out range is always empty
 // Can be specialized with an algorithm type if the behaviour is different, e.g. see set_union test.
 template<typename>
-int out_size_with_empty_in2(int) { return 0; };
+int out_size_with_empty_in2(int) { return 0; }
 
 auto data_gen2_default = [](auto i) { return i % 5 ? i : 0;};
 auto data_gen_unprocessed = [](auto) { return -1;};
@@ -359,9 +364,13 @@ private:
             test_dangling_pointers_args_3<idx>(std::forward<Policy>(exec), std::forward<Algo>(algo), std::forward<decltype(args)>(args)...);
     }
 
+    template<typename Policy, typename Algo, typename Checker, typename TransIn>
     void
-    process_data_in(int max_n, auto&& exec, auto algo, auto& checker, auto tr_in, auto... args)
+    process_data_in(int max_n, Policy&& exec, Algo algo, Checker& checker, TransIn tr_in, auto... args)
     {
+        std::string sizes{"for "};
+        sizes += std::to_string(max_n) + " elements";
+
         Container cont_in(exec, max_n, DataGen1{});
         Container cont_exp(exec, max_n, DataGen1{});
 
@@ -375,29 +384,31 @@ private:
         // check result types
         static_assert(std::is_same_v<decltype(res), decltype(expected_res)>, "Wrong return type");
 
-        using Algo = decltype(algo);
         EXPECT_EQ(ret_in_val(expected_res, expected_view.begin()), ret_in_val(res, r_in.begin()),
-                  (std::string("wrong return value from algo with ranges: ") + typeid(Algo).name() +
-                   typeid(decltype(tr_in(std::declval<Container&>()()))).name()).c_str());
+                  (std::string("wrong stop position with ") + typeid(Algo).name() +
+                   typeid(decltype(tr_in(std::declval<Container&>()()))).name() + sizes).c_str());
 
         //check result
         auto n = std::ranges::size(expected_view);
         if constexpr(is_range<std::remove_cvref_t<decltype(res)>>)
-            n = calc_res_size<std::remove_cvref_t<decltype(algo)>>(n, std::ranges::size(res));
+            n = calc_res_size<std::remove_cvref_t<Algo>>(n, std::ranges::size(res));
 
-        EXPECT_EQ_N(cont_exp().begin(), cont_in().begin(), n, (std::string("wrong effect algo with ranges: ")
-            + typeid(Algo).name() + typeid(decltype(tr_in(std::declval<Container&>()()))).name()).c_str());
+        EXPECT_EQ_N(cont_exp().begin(), cont_in().begin(), n, (std::string("data mismatch with ")
+            + typeid(Algo).name() + typeid(decltype(tr_in(std::declval<Container&>()()))).name() + sizes).c_str());
 
         // Test dangling iterators in return types for call with temporary data
         test_dangling_pointers<1, 100>(exec, algo, std::forward<decltype(args)>(args)...);
     }
 
-    template<typename Policy, typename Algo, typename Checker, typename TransIn, typename TransOut, TestDataMode mode = test_mode>
+    template<typename Policy, typename Algo, typename Checker, typename TransIn, typename TransOut,
+             TestDataMode mode = test_mode>
     void
     process_data_in_out(int max_n, int n_in, int n_out, Policy&& exec, Algo algo, Checker& checker, TransIn tr_in,
                         TransOut tr_out, auto... args)
     {
         static_assert(mode == data_in_out || mode == data_in_out_lim);
+        std::string sizes{"for "};
+        sizes += std::to_string(n_in) + " elements and " + std::to_string(n_out) + " space";
 
         Container cont_in(exec, n_in, DataGen1{});
         Container cont_in_exp(exec, n_in, DataGen1{});
@@ -421,18 +432,20 @@ private:
         static_assert(std::is_same_v<decltype(res), decltype(expected_res)>, "Wrong return type");
 
         EXPECT_EQ(ret_in_val(expected_res, in_exp_view.begin()), ret_in_val(res, tr_in(A).begin()),
-                  (std::string("wrong return value from algo with input range: ") + typeid(Algo).name()).c_str());
+                  (std::string("wrong input stop position with ") + typeid(Algo).name() + sizes).c_str());
 
         EXPECT_EQ(ret_out_val(expected_res, out_exp_view.begin()), ret_out_val(res, tr_out(B).begin()),
-                  (std::string("wrong return value from algo with output range: ") + typeid(Algo).name()).c_str());
+                  (std::string("wrong output stop position with ") + typeid(Algo).name() + sizes).c_str());
 
         //check result
         auto n = std::ranges::size(out_exp_view);
-        EXPECT_EQ_N(cont_out_exp().begin(), cont_out().begin(), n, (std::string("wrong effect algo with ranges: ") + typeid(Algo).name()).c_str());
+        EXPECT_EQ_N(cont_out_exp().begin(), cont_out().begin(), n, 
+                    (std::string("output mismatch with ") + typeid(Algo).name() + sizes).c_str());
 
         //check result
         auto n_in_exp = std::ranges::size(in_exp_view);
-        EXPECT_EQ_N(cont_in_exp().begin(), cont_in().begin(), n_in_exp, (std::string("wrong effect algo with ranges: ") + typeid(Algo).name()).c_str());
+        EXPECT_EQ_N(cont_in_exp().begin(), cont_in().begin(), n_in_exp,
+                    (std::string("input mismatch with ") + typeid(Algo).name() + sizes).c_str());
 
         // Test dangling iterators in return types for call with temporary data
         test_dangling_pointers<2, 200>(exec, algo, std::forward<decltype(args)>(args)...);
@@ -463,9 +476,16 @@ public:
 
         //test cases with empty sequence(s)
         process_data_in_out(max_n, 0, 0, CLONE_TEST_POLICY(exec), algo, checker, args...);
+        process_data_in_out(max_n, r_size, 0, CLONE_TEST_POLICY(exec), algo, checker, args...);
+        process_data_in_out(max_n, 0, r_size, CLONE_TEST_POLICY(exec), algo, checker, args...);
+
+        //test cases with only one element
+        process_data_in_out(max_n, r_size, 1, CLONE_TEST_POLICY(exec), algo, checker, args...);
+        process_data_in_out(max_n, 1, r_size, CLONE_TEST_POLICY(exec), algo, checker, args...);
     }
 
-    template<typename Policy, typename Algo, typename Checker, typename TransIn, typename TransOut, TestDataMode mode = test_mode>
+    template<typename Policy, typename Algo, typename Checker, typename TransIn, typename TransOut, 
+             TestDataMode mode = test_mode>
     std::enable_if_t<mode == data_in_in>
     operator()(int max_n, Policy&& exec, Algo algo, Checker& checker, TransIn tr_in, TransOut, auto... args)
     {
@@ -481,9 +501,14 @@ public:
     }
 
 private:
+    template<typename Policy, typename Algo, typename Checker, typename TransIn>
     void
-    process_data_in_in(int max_n, int n_in1, int n_in2, auto&& exec, auto algo, auto& checker, auto tr_in, auto... args)
+    process_data_in_in(int max_n, int n_in1, int n_in2, Policy&& exec, Algo algo, Checker& checker, TransIn tr_in,
+                       auto... args)
     {
+        std::string sizes{"for "};
+        sizes += std::to_string(n_in1) + " and " + std::to_string(n_in2) + " elements";
+
         assert(n_in1 <= max_n);
         assert(n_in2 <= max_n);
 
@@ -505,26 +530,29 @@ private:
         if constexpr (!std::is_same_v<decltype(res), bool>)
         {
             EXPECT_EQ(ret_in_val(expected_res, src_view1.begin()), ret_in_val(res, tr_in(A).begin()),
-                      (std::string("wrong return value from algo: ") + typeid(decltype(algo)).name() +
-                       typeid(decltype(tr_in(std::declval<Container&>()()))).name()).c_str());
+                      (std::string("wrong stop position with ") + typeid(Algo).name() +
+                       typeid(decltype(tr_in(std::declval<Container&>()()))).name() + sizes).c_str());
         }
         else
         {
             EXPECT_EQ(expected_res, res,
-                      (std::string("wrong return value from algo: ") + typeid(decltype(algo)).name() +
-                       typeid(decltype(tr_in(std::declval<Container&>()()))).name()).c_str());
+                      (std::string("wrong return value from ") + typeid(Algo).name() +
+                       typeid(decltype(tr_in(std::declval<Container&>()()))).name() + sizes).c_str());
         }
 
         // Test dangling iterators in return types for call with temporary data
         test_dangling_pointers<2, 300>(exec, algo, std::forward<decltype(args)>(args)...);
     }
 
-    template<typename Policy, typename Algo, typename Checker, typename TransIn, typename TransOut, TestDataMode mode = test_mode>
+    template<typename Policy, typename Algo, typename Checker, typename TransIn, typename TransOut,
+             TestDataMode mode = test_mode>
     void
     process_data_in_in_out(int max_n, int n_in1, int n_in2, int n_out, Policy&& exec, Algo algo, Checker& checker,
                            TransIn tr_in, TransOut tr_out, auto... args)
     {
         static_assert(mode == data_in_in_out || mode == data_in_in_out_lim);
+        std::string sizes{"for "};
+        sizes += std::to_string(n_in1) + " and " + std::to_string(n_in2) + " elements and " + std::to_string(n_out) + " space";
 
         Container cont_in1(exec, n_in1, DataGen1{});
         Container cont_in2(exec, n_in2, DataGen2{});
@@ -550,18 +578,18 @@ private:
         static_assert(std::is_same_v<decltype(res), decltype(expected_res)>, "Wrong return type");
 
         EXPECT_EQ(ret_in_val(expected_res, src_view1.begin()), ret_in_val(res, tr_in(A).begin()),
-                  (std::string("wrong return value from algo with input range 1: ") + typeid(Algo).name()).c_str());
+                  (std::string("wrong first input stop position with ") + typeid(Algo).name() + sizes).c_str());
 
         EXPECT_EQ(ret_in_val(expected_res, src_view2.begin()), ret_in_val(res, tr_in(B).begin()),
-                  (std::string("wrong return value from algo with input range 2: ") + typeid(Algo).name()).c_str());
+                  (std::string("wrong second input stop position with ") + typeid(Algo).name() + sizes).c_str());
 
         EXPECT_EQ(ret_out_val(expected_res, expected_view.begin()), ret_out_val(res, tr_out(C).begin()),
-                  (std::string("wrong return value from algo with output range: ") + typeid(Algo).name()).c_str());
+                  (std::string("wrong output stop position with ") + typeid(Algo).name() + sizes).c_str());
 
         //check result
         auto n = std::ranges::size(expected_view);
-        EXPECT_EQ_N(cont_exp().begin(), cont_out().begin(), n, (std::string("wrong effect algo with ranges: ") + typeid(Policy).name()
-            + typeid(Algo).name()).c_str());
+        EXPECT_EQ_N(cont_exp().begin(), cont_out().begin(), n, (std::string("output mismatch with ")
+                    + typeid(Algo).name() + typeid(Policy).name() + sizes).c_str());
 
         // Test dangling iterators in return types for call with temporary data
         test_dangling_pointers<3, 400>(exec, algo, std::forward<decltype(args)>(args)...);
@@ -824,9 +852,9 @@ template<int call_id = 0, typename T = int, TestDataMode mode = data_in, typenam
 struct test_range_algo
 {
     const int n_serial = small_size;
-    const int n_parallel = small_size;
+    const int n_parallel = medium_size/8; // 16K
 #if TEST_DPCPP_BACKEND_PRESENT
-    const int n_device = small_size;
+    const int n_device = medium_size;
 #endif
 
     test_range_algo() = default;
@@ -921,7 +949,7 @@ struct test_range_algo
     }
 };
 
-}; //namespace test_std_ranges
+} //namespace test_std_ranges
 
 #endif //_ENABLE_STD_RANGES_TESTING
 
