@@ -30,7 +30,7 @@ main()
             std::ranges::range_size_t<decltype(r_out)>>;
         Size size = std::ranges::min((Size)std::ranges::size(r_out), (Size)std::ranges::size(r_in));
 
-        auto res = std::ranges::transform(std::ranges::take_view(r_in, size),
+        std::ranges::transform(std::ranges::take_view(r_in, size),
             std::ranges::take_view(r_out, size).begin(), std::forward<decltype(args)>(args)...);
 
         using ret_type = std::ranges::unary_transform_result<std::ranges::borrowed_iterator_t<decltype(r_in)>,
@@ -56,7 +56,7 @@ main()
         if constexpr(std::ranges::sized_range<decltype(r_2)>)
             size = std::ranges::min(size, (Size)std::ranges::size(r_2));
 
-        auto res = std::ranges::transform(std::ranges::subrange(std::ranges::begin(r_1), std::ranges::begin(r_1) + size),
+        std::ranges::transform(std::ranges::subrange(std::ranges::begin(r_1), std::ranges::begin(r_1) + size),
             std::ranges::subrange(std::ranges::begin(r_2), std::ranges::begin(r_2) + size),
             std::ranges::take_view(r_out, size).begin(), std::forward<decltype(args)>(args)...);
 
@@ -66,7 +66,7 @@ main()
     };
 
     test_range_algo<4, int, data_in_in_out_lim>{big_sz}(dpl_ranges::transform, transform_binary_checker, binary_f);
-    test_range_algo<5, int, data_in_in_out_lim>{}(dpl_ranges::transform, transform_binary_checker, binary_f, proj, proj);
+    test_range_algo<5, int, data_in_in_out_lim>{}(dpl_ranges::transform, transform_binary_checker, binary_f, proj);
     test_range_algo<6, P2, data_in_in_out_lim>{}(dpl_ranges::transform, transform_binary_checker, binary_f, &P2::x, &P2::x);
     test_range_algo<7, P2, data_in_in_out_lim>{}(dpl_ranges::transform, transform_binary_checker, binary_f, &P2::proj, &P2::proj);
 #endif //_ENABLE_STD_RANGES_TESTING
