@@ -39,6 +39,10 @@
 #include <random>
 #include <limits>
 #include <cassert>
+#include <functional> // for std::invoke
+#if _ENABLE_STD_RANGES_TESTING
+#    include <ranges> // for std::ranges::enable_borrowed_range
+#endif
 
 #include "utils_const.h"
 #include "iterator_utils.h"
@@ -53,8 +57,8 @@
 namespace TestUtils
 {
 
-typedef double float64_t;
-typedef float float32_t;
+using float64_t = double;
+using float32_t = float;
 
 template <class T, ::std::size_t N>
 constexpr size_t
@@ -739,7 +743,7 @@ class Wrapper
     get_my_field() const
     {
         return my_field.get();
-    };
+    }
     static size_t
     Count()
     {
@@ -1340,6 +1344,97 @@ struct NoDefaultCtorWrapper {
     }
 };
 
+#if _ENABLE_STD_RANGES_TESTING
+
+// A minimalistic range: it can't be applied directly to hetero range-based algorithms,
+// but can be used for internal checks
+// Note, begin() and end() functions are still required, but they can be provided as ADL-discoverable free functions.
+template <typename RandomIt>
+struct MinimalisticRange
+{
+    RandomIt it_begin;
+    RandomIt it_end;
+
+#if TEST_STD_RANGES_VIEW_CONCEPT_REQUIRES_DEFAULT_INITIALIZABLE
+    MinimalisticRange() = default;
+#endif
+
+    MinimalisticRange(RandomIt it_begin, RandomIt it_end)
+        : it_begin(it_begin), it_end(it_end)
+    {
+    }
+};
+
+template <typename RandomIt>
+RandomIt
+begin(MinimalisticRange<RandomIt> range)
+{
+    return range.it_begin;
+}
+
+template <typename RandomIt>
+RandomIt
+end(MinimalisticRange<RandomIt> range)
+{
+    return range.it_end;
+}
+
+// A minimalistic view to detect an accidental use of methods such as begin(), end(), empty(), operator[] and etc,
+// which are not required for a class to be considered a range.
+template <typename RandomIt>
+struct MinimalisticView : MinimalisticRange<RandomIt>, std::ranges::view_base
+{
+#if TEST_STD_RANGES_VIEW_CONCEPT_REQUIRES_DEFAULT_INITIALIZABLE
+    MinimalisticView() = default;
+#endif
+
+    MinimalisticView(RandomIt it_begin, RandomIt it_end)
+        : MinimalisticRange<RandomIt>(it_begin, it_end)
+    {
+    }
+};
+
+template <typename RandomIt>
+RandomIt
+begin(MinimalisticView<RandomIt> view)
+{
+    return view.it_begin;
+}
+
+template <typename RandomIt>
+RandomIt
+end(MinimalisticView<RandomIt> view)
+{
+    return view.it_end;
+}
+
+#endif // _ENABLE_STD_RANGES_TESTING
+
+struct DefaultInitializedToOne
+{
+    int value = 1;
+
+    friend bool
+    operator==(const DefaultInitializedToOne& x, const DefaultInitializedToOne& y)
+    {
+        return x.value == y.value;
+    }
+};
+
 } /* namespace TestUtils */
 
+#if _ENABLE_STD_RANGES_TESTING
+
+// Standard-conforming specialization: inside namespace std::ranges
+namespace std
+{
+namespace ranges
+{
+template <typename RandomIt>
+inline constexpr bool enable_borrowed_range<TestUtils::MinimalisticRange<RandomIt>> = true;
+} // namespace ranges
+} // namespace std
+
+#endif // _ENABLE_STD_RANGES_TESTING
+
 #endif // _UTILS_H
