@@ -42,7 +42,7 @@ void test_edges()
         for (unsigned j = 0; j < N; ++j)
         {
             dpl::complex<double> r = dpl::pow(testcases[i], testcases[j]);
-            dpl::complex<double> z = dpl::exp(testcases[j] * dpl::log(testcases[i]));
+            [[maybe_unused]] dpl::complex<double> z = dpl::exp(testcases[j] * dpl::log(testcases[i]));
             if (std::isnan(dpl::real(r)))
             {
 #if !_PSTL_ICC_TEST_COMPLEX_POW_COMPLEX_COMPLEX_PASS_BROKEN_TEST_EDGES   // testcases[0], testcases[13]
