@@ -18,12 +18,10 @@
 #include <oneapi/dpl/iterator>
 #include <oneapi/dpl/numeric>
 
-#include <CL/sycl.hpp>
 #include <iostream>
-#include "support/test_iterators.h"
 
-constexpr sycl::access::mode sycl_read = sycl::access::mode::read;
-constexpr sycl::access::mode sycl_write = sycl::access::mode::write;
+#include "support/utils.h"
+#include "support/test_iterators.h"
 
 template <typename T1, typename T2>
 class KernelName;
@@ -40,7 +38,7 @@ template <class Iter1, class Iter2>
 void
 test()
 {
-    sycl::queue deviceQueue;
+    sycl::queue deviceQueue = TestUtils::get_test_queue();
     int input1[6] = {1, 2, 3, 4, 5, 6};
     int input2[6] = {6, 5, 4, 3, 2, 1};
     int output[8] = {};
@@ -52,9 +50,9 @@ test()
         sycl::buffer<int, 1> buffer2(input2, numOfItems1);
         sycl::buffer<int, 1> buffer3(output, numOfItems2);
         deviceQueue.submit([&](sycl::handler& cgh) {
-            auto in1 = buffer1.get_access<sycl_read>(cgh);
-            auto in2 = buffer2.get_access<sycl_read>(cgh);
-            auto out = buffer3.get_access<sycl_write>(cgh);
+            auto in1 = buffer1.get_access<sycl::access::mode::read>(cgh);
+            auto in2 = buffer2.get_access<sycl::access::mode::read>(cgh);
+            auto out = buffer3.get_access<sycl::access::mode::write>(cgh);
             cgh.single_task<KernelName<Iter1, Iter2>>([=]() {
                 out[0] = oneapi::dpl::inner_product(Iter1(&in1[0]), Iter1(&in1[0]), Iter2(&in2[0]), 1,
                                                     oneapi::dpl::multiplies<int>(), oneapi::dpl::plus<int>());
@@ -117,7 +115,5 @@ main()
     test<const int*, random_access_iterator<const int*>>();
     test<const int*, const int*>();
 #endif
-
-    std::cout << "done" << std::endl;
-    return 0;
+    return TestUtils::done();
 }
