@@ -14,14 +14,12 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include <CL/sycl.hpp>
-#include <iostream>
 #include <oneapi/dpl/numeric>
 
-#include "support/test_iterators.h"
+#include <iostream>
 
-constexpr sycl::access::mode sycl_read = sycl::access::mode::read;
-constexpr sycl::access::mode sycl_write = sycl::access::mode::write;
+#include "support/utils.h"
+#include "support/test_iterators.h"
 
 template <class T> class KernelTest;
 
@@ -32,14 +30,14 @@ template <typename _T1, typename _T2> void ASSERT_EQUAL(_T1 &&X, _T2 &&Y) {
 }
 
 template <class InIter> void test() {
-  sycl::queue deviceQueue;
+  sycl::queue deviceQueue = TestUtils::get_test_queue();
   int output[5] = {1, 2, 3, 4, 5};
   sycl::range<1> numOfItems1{5};
 
   {
     sycl::buffer<int, 1> buffer1(output, numOfItems1);
     deviceQueue.submit([&](sycl::handler &cgh) {
-      auto out = buffer1.get_access<sycl_write>(cgh);
+      auto out = buffer1.get_access<sycl::access::mode::write>(cgh);
       cgh.single_task<class KernelTest<InIter>>([=]() {
         oneapi::dpl::iota(InIter(&out[0]), InIter(&out[0] + 5), 5);
       });
@@ -56,6 +54,5 @@ int main() {
   test<bidirectional_iterator<int *>>();
   test<random_access_iterator<int *>>();
   test<int *>();
-  std::cout << "done" << std::endl;
-  return 0;
+  return TestUtils::done();
 }
