@@ -5,12 +5,6 @@
 //
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
-// This file incorporates work covered by the following copyright and permission
-// notice:
-//
-// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
-// See https://llvm.org/LICENSE.txt for license information.
-//
 //===----------------------------------------------------------------------===//
 //
 // Abstract:
@@ -163,7 +157,7 @@ public:
         constexpr std::int32_t num_elems =
             oneapi::dpl::internal::type_traits_t<typename Engine::result_type>::num_elems == 0
                 ? 1
-                : oneapi::dpl::internal::type_traits_t<typename Engine::result_type>::num_elems;
+                : oneapi::dpl::internal::type_traits_t<result_type>::num_elems;
 
         // Random number generation
         {
@@ -237,7 +231,7 @@ public:
         constexpr std::int32_t num_elems =
             oneapi::dpl::internal::type_traits_t<typename oneapi::dpl::ranlux24_vec<N>::result_type>::num_elems == 0
                 ? 1
-                : oneapi::dpl::internal::type_traits_t<typename oneapi::dpl::ranlux24_vec<N>::result_type>::num_elems;
+                : oneapi::dpl::internal::type_traits_t<result_type>::num_elems;
 
         // Random number generation
         {
@@ -257,13 +251,13 @@ public:
                         engine1.discard(offset);
                         typename oneapi::dpl::ranlux24_vec<N>::result_type res0;
                         oneapi::dpl::ranlux24_vec<N> engine(engine1);
-                        auto eng = engine.base();
-                        res0 = engine();
+                        auto eng = engine.base().min();
+                        res0 = engine() + eng;
                         typename oneapi::dpl::ranlux24_vec<N>::result_type res1 = engine1();
                         std::int32_t is_inequal = 0;
                         for (std::int32_t i = 0; i < num_elems; ++i)
                         {
-                            if (res0[i] != res1[i])
+                            if (res0[i] != res1[i] + eng)
                             {
                                 is_inequal = 1;
                             }
@@ -363,9 +357,9 @@ public:
                         {
                             dpstd_acc[offset] += 1;
                         }
-                        typename Engine::result_type res0;
+                        result_type res0;
                         res0 = engine0();
-                        typename Engine::result_type res1 = engine1();
+                        result_type res1 = engine1();
                         if (res0 != res1)
                         {
                             dpstd_acc[offset] += 1;
@@ -465,12 +459,12 @@ public:
                         {
                             dpstd_acc[offset] += 1;
                         }
-                        typename oneapi::dpl::ranlux24::result_type res0;
+                        result_type res0;
                         oneapi::dpl::ranlux24 engine(engine1);
-                        auto eng = engine.base();
-                        res0 = engine();
-                        typename oneapi::dpl::ranlux24::result_type res1 = engine1();
-                        if (res0 != res1)
+                        auto eng = engine.base().min();
+                        res0 = engine() + eng;
+                        result_type res1 = engine1();
+                        if (res0 != res1 + eng)
                         {
                             dpstd_acc[offset] += 1;
                         }
@@ -511,7 +505,7 @@ main()
 #if TEST_UNNAMED_LAMBDAS
 
     sycl::queue queue = TestUtils::get_test_queue();
-    
+
     std::int32_t err = 0;
 
     std::cout << "---------------------------------------------------" << std::endl;
