@@ -5,12 +5,6 @@
 //
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 //
-// This file incorporates work covered by the following copyright and permission
-// notice:
-//
-// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
-// See https://llvm.org/LICENSE.txt for license information.
-//
 //===----------------------------------------------------------------------===//
 //
 // Abstract:
@@ -103,7 +97,7 @@ int test_portion(sycl::queue& queue, oneapi::dpl::internal::element_type_t<UIntT
                 oneapi::dpl::subtract_with_carry_engine<UIntType, w, s, r> engine(seed, offset);
 
                 auto res = engine(part);
-                for(int i = 0; i < n_elems; ++i)
+                for(unsigned int i = 0; i < n_elems; ++i)
                     dpstd_acc[offset + i] = res[i];
             });
         });
